@@ -40,6 +40,7 @@ import ftrack_api.accessor.disk
 import ftrack_api.structure.origin
 import ftrack_api.structure.entity_id
 import ftrack_api.accessor.server
+import ftrack_api._centralized_storage_scenario
 from ftrack_api.logging import LazyLogMessage as L
 
 
@@ -250,6 +251,8 @@ class Session(object):
         self.schemas = self._load_schemas(schema_cache_path)
         self.types = self._build_entity_type_classes(self.schemas)
 
+        ftrack_api._centralized_storage_scenario.register(self)
+
         self._configure_locations()
 
     @property
@@ -726,19 +729,21 @@ class Session(object):
 
     def _merge(self, value, merged):
         '''Return merged *value*.'''
+        log_debug = self.logger.isEnabledFor(logging.DEBUG)
+
         if isinstance(value, ftrack_api.entity.base.Entity):
-            self.logger.debug(L(
-                'Merging entity into session: {0} at {1}',
-                value, id(value)
-            ))
+            log_debug and self.logger.debug(
+                'Merging entity into session: {0} at {1}'
+                .format(value, id(value))
+            )
 
             return self._merge_entity(value, merged=merged)
 
         elif isinstance(value, ftrack_api.collection.Collection):
-            self.logger.debug(L(
-                'Merging collection into session: {0!r} at {1}',
-                value, id(value)
-            ))
+            log_debug and self.logger.debug(
+                'Merging collection into session: {0!r} at {1}'
+                .format(value, id(value))
+            )
 
             merged_collection = []
             for entry in value:
@@ -749,10 +754,10 @@ class Session(object):
             return merged_collection
 
         elif isinstance(value, ftrack_api.collection.MappedCollectionProxy):
-            self.logger.debug(L(
-                'Merging mapped collection into session: {0!r} at {1}',
-                value, id(value)
-            ))
+            log_debug and self.logger.debug(
+                'Merging mapped collection into session: {0!r} at {1}'
+                .format(value, id(value))
+            )
 
             merged_collection = []
             for entry in value.collection:
@@ -775,6 +780,8 @@ class Session(object):
         merged entity instance is used.
 
         '''
+        log_debug = self.logger.isEnabledFor(logging.DEBUG)
+
         if merged is None:
             merged = {}
 
@@ -786,29 +793,29 @@ class Session(object):
             # Check whether this entity has already been processed.
             attached_entity = merged.get(entity_key)
             if attached_entity is not None:
-                self.logger.debug(L(
-                    'Entity already processed for key {0} as {1} at {2}',
-                    entity_key, attached_entity, id(attached_entity)
-                ))
+                log_debug and self.logger.debug(
+                    'Entity already processed for key {0} as {1} at {2}'
+                    .format(entity_key, attached_entity, id(attached_entity))
+                )
 
                 return attached_entity
             else:
-                self.logger.debug(L(
-                    'Entity not already processed for key {0}. Keys: {1}',
-                    entity_key, sorted(merged.keys())
-                ))
+                log_debug and self.logger.debug(
+                    'Entity not already processed for key {0}.'
+                    .format(entity_key)
+                )
 
             # Check for existing instance of entity in cache.
-            self.logger.debug(L(
-                'Checking for entity in cache with key {0}', entity_key
-            ))
+            log_debug and self.logger.debug(
+                'Checking for entity in cache with key {0}'.format(entity_key)
+            )
             try:
                 attached_entity = self.cache.get(entity_key)
                 from_cache = True
-                self.logger.debug(L(
-                    'Retrieved existing entity from cache: {0} at {1}',
-                    attached_entity, id(attached_entity)
-                ))
+                log_debug and self.logger.debug(
+                    'Retrieved existing entity from cache: {0} at {1}'
+                    .format(attached_entity, id(attached_entity))
+                )
 
             except KeyError:
                 # Construct new minimal instance to store in cache.
@@ -816,10 +823,10 @@ class Session(object):
                     entity.entity_type, {}, reconstructing=True
                 )
                 from_cache = False
-                self.logger.debug(L(
+                log_debug and self.logger.debug(
                     'Entity not present in cache. Constructed new instance: '
-                    '{0} at {1}', attached_entity, id(attached_entity)
-                ))
+                    '{0} at {1}'.format(attached_entity, id(attached_entity))
+                )
 
             # Mark entity as seen to avoid infinite loops.
             merged[entity_key] = attached_entity
@@ -870,6 +877,7 @@ class Session(object):
         else:
             entity._inflated = True
 
+        log_debug = self.logger.isEnabledFor(logging.DEBUG)
         self.logger.debug('Merging references.')
 
         if merged is None:
@@ -887,9 +895,9 @@ class Session(object):
                     ftrack_api.collection.MappedCollectionProxy
                 )
             ):
-                self.logger.debug(L(
-                    'Merging local value for attribute {0}.', attribute
-                ))
+                log_debug and self.logger.debug(
+                    'Merging local value for attribute {0}.'.format(attribute)
+                )
 
                 merged_local_value = self._merge(local_value, merged=merged)
                 if merged_local_value is not local_value:
@@ -906,9 +914,9 @@ class Session(object):
                     ftrack_api.collection.MappedCollectionProxy
                 )
             ):
-                self.logger.debug(L(
-                    'Merging remote value for attribute {0}.', attribute
-                ))
+                log_debug and self.logger.debug(
+                    'Merging remote value for attribute {0}.'.format(attribute)
+                )
 
                 merged_remote_value = self._merge(remote_value, merged=merged)
                 if merged_remote_value is not remote_value:
@@ -1472,6 +1480,23 @@ class Session(object):
         location.structure = ftrack_api.structure.entity_id.EntityIdStructure()
         location.priority = 150
 
+        # Master location based on server scenario.
+        storage_scenario = self.server_information.get('storage_scenario')
+
+        if (
+            storage_scenario and
+            storage_scenario.get('scenario')
+        ):
+            self.event_hub.publish(
+                ftrack_api.event.base.Event(
+                    topic='ftrack.storage-scenario.activate',
+                    data=dict(
+                        storage_scenario=storage_scenario
+                    )
+                ),
+                synchronous=True
+            )
+
         # Next, allow further configuration of locations via events.
         self.event_hub.publish(
             ftrack_api.event.base.Event(
@@ -2099,6 +2124,53 @@ class Session(object):
 
         return self.get('Job', result[0]['job_id'])
 
+    def send_review_session_invite(self, invitee):
+        '''Send an invite to a review session to *invitee*.
+
+        *invitee* is a instance of ReviewSessionInvitee.
+
+        .. note::
+
+            The *invitee* must be committed.
+
+        '''
+        self.send_review_session_invites([invitee])
+
+    def send_review_session_invites(self, invitees):
+        '''Send an invite to a review session to a list of *invitees*.
+
+        *invitee* is a list of ReviewSessionInvitee objects.
+
+        .. note::
+
+            All *invitees* must be committed.
+
+        '''
+        operations = []
+
+        for invitee in invitees:
+            operations.append(
+                {
+                    'action': 'send_review_session_invite',
+                    'review_session_invitee_id': invitee['id']
+                }
+            )
+
+        try:
+            self._call(operations)
+        except ftrack_api.exception.ServerError as error:
+            # Raise informative error if the action is not supported.
+            if 'Invalid action u\'send_review_session_invite\'' in error.message:
+                raise ftrack_api.exception.ServerCompatibilityError(
+                    'Server version {0!r} does not support '
+                    '"send_review_session_invite", please update server and '
+                    'try again.'.format(
+                        self.server_information.get('version')
+                    )
+                )
+            else:
+                raise
+
 
 class AutoPopulatingContext(object):
     '''Context manager for temporary change of session auto_populate value.'''
